require('dotenv').config()
const express = require('express');
const path = require('path');
const url = require('url')
const app = express();
const client = require('prom-client');

const collectDefaultMetrics = client.collectDefaultMetrics;
const register = new client.Registry();
client.collectDefaultMetrics({
    app: 'url',
    prefix: 'node_',
    timeout: 10000,
    gcDurationBuckets: [0.001, 0.01, 0.1, 1, 2, 5],
    register
});

const httpRequestTimer = new client.Histogram({
    name: 'http_request_duration_seconds',
    help: 'Duration of HTTP requests in seconds',
    labelNames: ['method', 'route', 'code'],
    buckets: [0.1, 0.3, 0.5, 0.7, 1, 3, 5, 7, 10] // 0.1 to 10 seconds
});

const createDelayHandler = async (req, res) => {
    if ((Math.floor(Math.random() * 100)) === 0) {
        throw new Error('Internal Error')
    }

    // delay for 3-6 seconds
    const delaySeconds = Math.floor(Math.random() * (6 - 3)) + 3
    await new Promise(res => setTimeout(res, delaySeconds * 1000))

    res.end('Slow url accessed !!');
};

register.registerMetric(httpRequestTimer);

// Database config
const connection = require('./config/db.config');
connection.once('open', () => console.log('DB Connected'));
connection.on('error', () => console.log('Error'));

// Routes Config
app.use(express.json({
    extended: false
})); //parse incoming request body in JSON format.

app.get('/metrics', async (req, res) => {
    // Start the timer
    const end = httpRequestTimer.startTimer();
    const route = req.route.path;

    res.setHeader('Content-Type', register.contentType);
    res.send(await register.metrics());

    // End timer and add labels
    end({ route, code: res.statusCode, method: req.method });
});

app.get('/slow', async (req, res) => {
    // Start the timer
    const end = httpRequestTimer.startTimer();
    const route = req.route.path;

    await createDelayHandler(req, res);

    // End timer and add labels
    end({ route, code: res.statusCode, method: req.method });
});

app.use('/', require('./service/redirect'));
app.use('/api/url', require('./service/url'));
app.get('/', function(req, res) {
    res.sendFile(path.join(__dirname + `../../public/index.html`))
});
app.get('/public/js/app.js', function(req, res) {
    res.sendFile(path.join(__dirname + `../../public/js/app.js`))
});
app.get('/public/js/service/FetchService.js', function(req, res) {
    res.sendFile(path.join(__dirname + `../../public/js/service/FetchService.js`));
});
app.get('/public/main.css', function(req, res) {
    res.sendFile(path.join(__dirname + `../../public/main.css`));
});

//Listen for incoming requests
const PORT = process.env.PORT;
app.listen(PORT, console.log(`server started, listening PORT ${PORT}`));
console.log(`Access Server at: https://localhost:${PORT}`);
